'use strict';

var tlsUtils = require('../tls/tlsUtils');
var http = require('http');
var config = require('../common/config');
var colors = require('colors');
var createRequestHandler = require('./createRequestHandler');
var createConnectHandler = require('./createConnectHandler');
var createFakeServerCenter = require('./createFakeServerCenter');
var createUpgradeHandler = require('./createUpgradeHandler');

module.exports = {
    createProxy: function createProxy(_ref) {
        var _ref$port = _ref.port;
        var port = _ref$port === undefined ? config.defaultPort : _ref$port;
        var caCertPath = _ref.caCertPath;
        var caKeyPath = _ref.caKeyPath;
        var sslConnectInterceptor = _ref.sslConnectInterceptor;
        var requestInterceptor = _ref.requestInterceptor;
        var responseInterceptor = _ref.responseInterceptor;
        var getCertSocketTimeout = _ref.getCertSocketTimeout;
        var _ref$rejectUnauthoriz = _ref.rejectUnauthorized;
        var rejectUnauthorized = _ref$rejectUnauthoriz === undefined ? false : _ref$rejectUnauthoriz;
        var _ref$plugins = _ref.plugins;
        var plugins = _ref$plugins === undefined ? [] : _ref$plugins;


        // Don't reject unauthorized
        if (!rejectUnauthorized) {
            process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = '0';
        }

        if (!caCertPath && !caKeyPath) {
            var rs = this.createCA();
            caCertPath = rs.caCertPath;
            caKeyPath = rs.caKeyPath;
            if (rs.create) {
                console.log(colors.cyan('CA Cert saved in: ' + caCertPath));
                console.log(colors.cyan('CA private key saved in: ' + caKeyPath));
            }
        }

        port = ~ ~port;
        var requestHandler = createRequestHandler(requestInterceptor, responseInterceptor, plugins);

        var upgradeHandler = createUpgradeHandler();

        var fakeServersCenter = createFakeServerCenter({
            caCertPath: caCertPath,
            caKeyPath: caKeyPath,
            requestHandler: requestHandler,
            upgradeHandler: upgradeHandler,
            getCertSocketTimeout: getCertSocketTimeout
        });

        var connectHandler = createConnectHandler(sslConnectInterceptor, fakeServersCenter);

        var server = new http.Server();
        server.listen(port, function () {
            console.log(colors.green('node-mitmproxy启动端口: ' + port));
            server.on('error', function (e) {
                console.error(colors.red(e));
            });
            server.on('request', function (req, res) {
                var ssl = false;
                requestHandler(req, res, ssl);
            });
            // tunneling for https
            server.on('connect', function (req, cltSocket, head) {
                connectHandler(req, cltSocket, head);
            });
            // TODO: handler WebSocket
            server.on('upgrade', function (req, socket, head) {
                var ssl = false;
                upgradeHandler(req, socket, head, ssl);
            });
        });
    },
    createCA: function createCA() {
        var caBasePath = arguments.length <= 0 || arguments[0] === undefined ? config.getDefaultCABasePath() : arguments[0];

        return tlsUtils.initCA(caBasePath);
    }
};